% this file demos how to recover the sparse part from gross innovation
% some interesting results: we get 920/1300 mis-classify with estimated
% sparse part and 350/1300 mis-classify with learned sparse part.
function GrossInnovation
clear all;
close all;
cd('E:\Documents\MATLAB\AR\');
try
    load('AR-SRC.mat');
catch
    %% load in the dictionary
    load('AR-train.mat', 'A_hat', 'C_hat', 'E_hat', 'option');
    % convert the sparse part into gross innovation
    Gi=zeros(size(C_hat));
    Gj=zeros(size(A_hat));
    for i=1: 50
        for j=1: 13
            Gi(:, (i-1)*40+1: 40*i)=reshape(mean(reshape(E_hat(55*(i-1)+1: 55*i, 1: 520), 55*40, []), 2), 55, 40);
            Gj(:, (j-1)*40+1: 40*j)=reshape(mean(reshape(E_hat(1: 2750, 40*(j-1)+1: 40*j)', 55*40, []), 2), 40, 55)';
        end
    end
    for i=51: 100
        for j=14: 26
            Gi(:, (i-1)*40+1: 40*i)=reshape(mean(reshape(E_hat(55*(i-1)+1: 55*i, 1: 520), 55*40, []), 2), 55, 40);
            Gj(:, (j-1)*40+1: 40*j)=reshape(mean(reshape(E_hat(2751: 5500, 40*(j-14)+1: 40*(j-13))', 55*40, []), 2), 40, 55)';
        end
    end
    clear i j;

    %% recover the sparse part
    E_tmp=zeros(size(E_hat));
    for i=1: 50
        for j=1: 13
            E_tmp(55*(i-1)+1: 55*i, 40*(j-1)+1: 40*j)=ThreshJSM(Gi(:, (i-1)*40+1: 40*i), Gj(:, (j-1)*40+1: 40*j));
        end
    end
    for i=51: 100
        for j=14: 26
            E_tmp(55*(i-1)+1: 55*i, 40*(j-14)+1: 40*(j-13))=ThreshJSM(Gi(:, (i-1)*40+1: 40*i), Gj(:, (j-1)*40+1: 40*j));
        end
    end
    % E_hat=E_tmp;
    clear i j E_tmp;

    %% build the dictionary
    X=zeros(55*40, 100*13);
    for i=1: 50
        for j=1: 13
            X(:, (i-1)*13+j)=reshape(E_hat(55*(i-1)+1: 55*i, 40*(j-1)+1: 40*j)+C_hat(:, (i-1)*40+1: 40*i)+A_hat(:, (j-1)*40+1: 40*j), [], 1);
        end
    end
    for i=51: 100
        for j=14: 26
            X(:, (i-2)*13+j)=reshape(E_hat(55*(i-1)+1: 55*i, 40*(j-14)+1: 40*(j-13))+C_hat(:, (i-1)*40+1: 40*i)+A_hat(:, (j-1)*40+1: 40*j), [], 1);
        end
    end

    %% do normalization
    X=X./repmat(sqrt(sum(X.^2, 1)), 2200, 1);
    save('AR-SRC.mat', 'X');
end

%% some parameters for GPSR
% tau = 0.009;
% steps = 5;
% debias = 0;
% stopCri=3;
% tolA=1e-3;

%% try SRC method to evaluate the performance of the sparse part
% cd('E:\Documents\MATLAB\AR\test');
dirlist=dir('*.bmp');
idx=zeros(1, length(dirlist));
err=zeros(100, length(dirlist));
coef=zeros(1300, length(dirlist));
for i=1: length(dirlist)
    img=imread(dirlist(i).name);
    img=rgb2gray(img);
    img=imresize(img, [55, 40]);
    img=double(img(:));
    % use omp
    coef(:, i)=OMP(X, img, 26);
%     x0=X'*img;
%     first_tau_factor = 0.8*(max (max((abs(x0)./tau))));
%     coef(:, i)=GPSR_BB(img, X, tau,...
%         'Debias',0,...
%         'Continuation',1,...
%         'ContinuationSteps',steps,...
%         'FirstTauFactor',first_tau_factor,...
%         'Monotone',1,...
%         'Initialization',x0,...
%         'StopCriterion',3,...
%         'ToleranceA',tolA,...
%         'Verbose',0);
    % calculate the recontruction error
    for j=1: 100
        err(j, i)=norm(img-X(:, 13*(j-1)+1: 13*j)*coef(13*(j-1)+1: 13*j, i), 2);
    end
end

%% summarize the result
[val, idx]=min(err);
lbl=ceil([1: 1300]/13);
val=find(idx~=lbl);