% V1:   the classification is based on the principle angle between two
% subspaces, i.e. C_i+{A_j} and X+{A_j}
% parameters:
% file_name:    the image name for test;
% C_hat:        the common part, where each column is for one subject;
% A_hat:        the low rank part, where each column is for one condition;
% G_hat:        not used in this version;
% option:       a structure contains many parameter;
% =========================================================================
function [idx, output]=TestSubspace1(file_name, C_hat, A_hat, G_hat, option)
%% ========================================================================
% preprocessing
% number of images
N=2;
% read in the image
img=imread(file_name);
% color image?
if size(img, 3)~=1
    img=0.2989*img(:, :, 1)+0.5870*img(:, :, 2)+0.1140*img(:, :, 3);
end
% resize the image?
if option.height~=0 && option.width~=0
    img=imresize(img, [option.height, option.width]);
end
height=size(img, 1);
width=size(img, 2);
% some preprocessing may be need here
img=double(img);
% normalization
switch option.mode
    case 0
        % use the logrithm
        img=log(img+1);
    case 1
        % unit l2-norm for each image
        img=img/norm(img, 'fro');
    case 2
        % zero mean and unit l2-norm for each image
        img=img-mean(img(:));
        img=img/norm(img, 'fro');
    case 3
        % zero mean and unit std for eac dimension
        img=img-mean(img(:));
        img=img/std(img(:));
    case 4
        % zerp mean
        img=img-mean(img(:));
end
%% ========================================================================
% Parameters
X=repmat(img(:), 1, size(A_hat, 2))+A_hat;
block=zeros(option.n_People, 25);
coef=zeros(option.n_People, size(A_hat, 2));
% =========================================================================
parfor i=1: size(C_hat, 2)
    coef(i, :)=subspacea(repmat(C_hat(:, i), 1, size(A_hat, 2))+A_hat, X);
    block(i, :)=BlockError(repmat(C_hat(:, i), 1, size(A_hat, 2))+A_hat, X, option);
end
output.coef=coef;
output.block=block;
%% ========================================================================
% do classification according to the reconstruction error
if strcmpi(option.dataset, 'Multi-PIE')
    output.err=sum(cos(output.coef).^2, 2);
    [val, idx]=max(output.err);
    idx=mod(idx-1, option.n_People)+1;
    tmp=max(output.block);
    output.block=output.block./repmat(tmp, size(C_hat, 2), 1);
    output.block=sum(output.block, 2);
    [val, output.idx]=max(output.block);
    output.idx=mod(output.idx-1, option.n_People)+1;
else
    output.err=sum(cos(output.coef).^2, 2);
    [val, idx]=max(output.err);
    tmp=max(output.block);
    output.block=output.block./repmat(tmp, size(C_hat, 2), 1);
    output.block=sum(output.block, 2);
    [val, output.idx]=max(output.block);
end
end

% calculate the error in each block (1/2 overlapping), where mode decides 
% the error measurement we use:
% 1:    l1 norm;
% 2:    l2 norm;
function err=BlockError(X, Y, option)
err=zeros(1, 25);
nCol=size(X, 2);
X=reshape(X, option.height, []);
Y=reshape(Y, option.height, []);
tmpX=[];
tmpY=[];
tmpTheta=[];
l=1;
if strcmpi(option.dataset, 'AR')
    for i=[1 10 19 28 38]
        for j=[1 8 15 22 27]
            for k=1: nCol
                tmpX(:, k)=reshape(X(i: i+17, (k-1)*option.width+(j: j+13)), [], 1);
                tmpY(:, k)=reshape(Y(i: i+17, (k-1)*option.width+(j: j+13)), [], 1);
            end
            tmpTheta=subspacea(tmpX, tmpY);
            err(l)=sum(cos(tmpTheta).^2);
            l=l+1;
        end
    end
elseif strcmpi(option.dataset, 'YaleB')
    for i=[1 9 17 25 33]
        for j=[1 8 15 22 29]
            for k=1: nCol
                tmpX(:, k)=reshape(X(i: i+15, (k-1)*option.width+(j: j+13)), [], 1);
                tmpY(:, k)=reshape(Y(i: i+15, (k-1)*option.width+(j: j+13)), [], 1);
            end
            tmpTheta=subspacea(tmpX, tmpY);
            err(l)=sum(cos(tmpTheta).^2);
            l=l+1;
        end
    end
elseif strcmpi(option.dataset, 'PIE') || strcmpi(option.dataset, 'Multi_PIE')
    for i=[1 9 18 26 34]
        for j=[1 6 13 18 24]
            for k=1: nCol
                tmpX(:, k)=reshape(X(i: i+16, (k-1)*option.width+(j: j+11)), [], 1);
                tmpY(:, k)=reshape(Y(i: i+16, (k-1)*option.width+(j: j+11)), [], 1);
            end
            tmpTheta=subspacea(tmpX, tmpY);
            err(l)=sum(cos(tmpTheta).^2);
            l=l+1;
        end
    end
end
end