% V5:       this version we deal with missing training entries problem, we
%           use JointMC7 (set the weight of missing entries to be zero);
% V4:       We do not seperate the male and female in AR dataset.
% V3.2:     as V3.1
% V3.1:     A subversion of 3. We convert the image to DCT basis and then
%           convert back.
% V3:       We treat the male and female in AR dataset seperately in this
%           version
% V2:       this version uses the JSM 3 for seperate the images from each
%           subjects independently.
% V1:   RPCA
% parameters:
% dataset:  name of dataset
% path:     path of data file;
% i:        idx of people;
% =========================================================================
function [C_hat, A_hat, E_hat, output]=TrainJointMC5(dataset, path, option)
%% ========================================================================
if strcmpi(dataset, 'AR')
    % for AR dataset, we split the dataset into male and female parts and
    % traning them seperately.
    option.n_People=option.n_People/2;
    option.n_Condition=option.n_Condition/2;
    % =====================================================================
    % Male part
    omega=GenerateSample(option.n_People, option.n_Condition, option.ratio);
    % load in the images
    X=zeros(option.height*option.n_People, option.width*option.n_Condition);
    if option.debug==1
        f1=fopen('filelist.txt', 'w');
        fprintf(f1, 'People\tCondition\tFile\n');
    end
    cd(path);
    dirlist=dir('M*.bmp');
    img_mean=zeros(option.n_People, option.n_Condition);
    img_std=ones(option.n_People, option.n_Condition);
    for i=1: option.n_People
        for j=1: option.n_Condition
            if omega(i, j)==1
                img=imread(dirlist((i-1)*option.n_Condition+j).name);
                if size(img, 3)~=1
                    img=rgb2gray(img);
                end
                img=imresize(img, [option.height option.width]);
                img=double(img);
                % normalization needed
                switch option.mode
                    case 0
                        % use the logrithm
                        img=log(img+1);
                    case 1
                        % unit l2-norm for each image
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 2
                        % zero mean and unit l2-norm for each image
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 3
                        % zero mean and unit std for eac dimension
                        img_std(i, j)=255;
                        img=img/255;
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                    case 4
                        % zerp mean
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                end
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=img;
            else
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=0;
            end
            % for debug only
            if option.debug==1
                fprintf(f1, '%d\t\t%d\t\t%s\n', i, j, dirlist((i-1)*option.n_Condition+j).name);
            end
        end
    end
%     matrix2im(X, 'AR-X1.tif', 8);
    % start training
    % lambda=1/sqrt(size(X, 1));
    % use a smaller weight for sparisity term
    lambda=option.lambda/sqrt(option.height);
    % for debug only
    tstart=tic;
    % start program
    [C_hat, A_hat, E_hat, output]=JointMC7(X, [option.height, option.width, ...
        option.n_People, option.n_Condition], omega, lambda, option.tol, ...
        option.maxIter, option.limem, option.debug, ...
        option.tau, option.rho);
    tstop=toc(tstart);
    disp(['Time for JointMC7 is ', num2str(tstop)]);
    output.omega=omega;
    output.mean=img_mean;
    output.std=img_std;
    % =====================================================================
    % Female part
    omega=GenerateSample(option.n_People, option.n_Condition, option.ratio);
    % load in the images
    X=zeros(option.height*option.n_People, option.width*option.n_Condition);
    dirlist=dir('W*.bmp');
    img_mean=zeros(option.n_People, option.n_Condition);
    img_std=ones(option.n_People, option.n_Condition);
    for i=1: option.n_People
        for j=1: option.n_Condition
            if omega(i, j)==1
                img=imread(dirlist((i-1)*option.n_Condition+j).name);
                if size(img, 3)~=1
                    img=rgb2gray(img);
                end
                img=imresize(img, [option.height option.width]);
                img=double(img);
                % normalization needed
                switch option.mode
                    case 0
                        % use the logrithm
                        img=log(img+1);
                    case 1
                        % unit l2-norm for each image
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 2
                        % zero mean and unit l2-norm for each image
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 3
                        % zero mean and unit std for eac dimension
                        img_std(i, j)=255;
                        img=img/255;
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                    case 4
                        % zerp mean
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                end
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=img;
            else
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=0;
            end
            % for debug only
            if option.debug==1
                fprintf(f1, '%d\t\t%d\t\t%s\n', i, j, dirlist((i-1)*option.n_Condition+j).name);
            end
        end
    end
    if option.debug==1
        fclose(f1);
        clear f1;
    end
%     matrix2im(X, 'AR-X2.tif', 8);
    % start training
    % lambda=1/sqrt(size(X, 1));
    % use a smaller weight for sparisity term
    lambda=option.lambda/sqrt(option.height);
    % for debug only
    tstart=tic;
    % start program
    [tmp_C, tmp_A, tmp_E, tmpOutput]=JointMC7(X, [option.height, option.width, ...
        option.n_People, option.n_Condition], omega, lambda, option.tol, ...
        option.maxIter, option.limem, option.debug, ...
        option.tau, option.rho);
    tstop=toc(tstart);
    disp(['Time for JointMC7 is ', num2str(tstop)]);
    % =====================================================================
    % Output
    C_hat=[C_hat, tmp_C];
    A_hat=[A_hat, tmp_A];
    E_hat=[E_hat; tmp_E];
    tmpOutput.omega=omega;
    tmpOutput.mean=img_mean;
    tmpOutput.std=img_std;
    output=[output; tmpOutput];
    option.n_People=option.n_People*2;
    option.n_Condition=option.n_Condition*2;
    output(1).omega=[output(1).omega omega];
%% ========================================================================
elseif strcmpi(dataset, 'YaleB')
    % deciding the sampling entries
    omega=GenerateSample(option.n_People, option.n_Condition, option.ratio);
    % load in the images
    X=zeros(option.height*option.n_People, option.width*option.n_Condition);
    if option.debug==1
        f1=fopen('filelist.txt', 'w');
        fprintf(f1, 'People\tCondition\tFile\n');
    end
    cd(path);
    dirlist=dir('*.bmp');
    img_mean=zeros(option.n_People, option.n_Condition);
    img_std=ones(option.n_People, option.n_Condition);
    for i=1: option.n_People
        for j=1: option.n_Condition
            if omega(i, j)==1
                img=imread(dirlist((i-1)*option.n_Condition+j).name);
                if size(img, 3)~=1
                    img=rgb2gray(img);
                end
                img=imresize(img, [option.height option.width]);
                img=double(img);
                % normalization needed
                switch option.mode
                    case 0
                        % use the logrithm
                        img=log(img+1);
                    case 1
                        % unit l2-norm for each image
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 2
                        % zero mean and unit l2-norm for each image
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 3
                        % zero mean and unit std for eac dimension
                        img_std(i, j)=255;
                        img=img/255;
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                    case 4
                        % zerp mean
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                end
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=img;
            else
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=0;
            end
            % for debug only
            if option.debug==1
                fprintf(f1, '%d\t\t%d\t\t%s\n', i, j, dirlist((i-1)*option.n_Condition+j).name);
            end
        end
    end
    if option.debug==1
        fclose(f1);
        clear f1;
    end
%     matrix2im(X, 'YaleB-X.tif', 8);
    % start training
    % lambda=1/sqrt(size(X, 1));
    % use a smaller weight for sparisity term
    lambda=option.lambda/sqrt(option.height);
    % for debug only
    tstart=tic;
    % start program
    [C_hat, A_hat, E_hat, output]=JointMC7(X, [option.height, option.width, ...
        option.n_People, option.n_Condition], omega, lambda, option.tol, ...
        option.maxIter, option.limem, option.debug, ...
        option.tau, option.rho);
    tstop=toc(tstart);
    disp(['Time for JointMC7 is ', num2str(tstop)]);
    output.omega=omega;
    output.mean=img_mean;
    output.std=img_std;
elseif strcmpi(dataset, 'PIE')
    % note, subject 38, 33 are not observed
    % deciding the sampling entries
    omega=GenerateSample(option.n_People, option.n_Condition, option.ratio);
    % load in the images
    X=zeros(option.height*option.n_People, option.width*option.n_Condition);
    if option.debug==1
        f1=fopen('filelist.txt', 'w');
        fprintf(f1, 'People\tCondition\tFile\n');
    end
    cd(path);
    dirlist=dir('*.jpg');
    img_mean=zeros(option.n_People, option.n_Condition);
    img_std=ones(option.n_People, option.n_Condition);
    for i=1: option.n_People
        for j=1: option.n_Condition
            if omega(i, j)==1
                img=imread(dirlist((i-1)*option.n_Condition+j).name);
                if size(img, 3)~=1
                    img=rgb2gray(img);
                end
                img=imresize(img, [option.height option.width]);
                img=double(img);
                % normalization needed
                switch option.mode
                    case 0
                        % use the logrithm
                        img=log(img+1);
                    case 1
                        % unit l2-norm for each image
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 2
                        % zero mean and unit l2-norm for each image
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                        img_std(i, j)=norm(img, 'fro');
                        img=img/img_std(i, j);
                    case 3
                        % zero mean and unit std for eac dimension
                        img_std(i, j)=255;
                        img=img/255;
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                    case 4
                        % zerp mean
                        img_mean(i, j)=mean(img(:));
                        img=img-img_mean(i, j);
                end
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=img;
            else
                X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=0;
            end
            % for debug only
            if option.debug==1
                fprintf(f1, '%d\t\t%d\t\t%s\n', i, j, dirlist((i-1)*option.n_Condition+j).name);
            end
        end
    end
    if option.debug==1
        fclose(f1);
        clear f1;
    end
%     matrix2im(X, 'YaleB-X.tif', 8);
    % start training
    % lambda=1/sqrt(size(X, 1));
    % use a smaller weight for sparisity term
    lambda=option.lambda/sqrt(option.height);
    % for debug only
    tstart=tic;
    % start program
    [C_hat, A_hat, E_hat, output]=JointMC7(X, [option.height, option.width, ...
        option.n_People, option.n_Condition], omega, lambda, option.tol, ...
        option.maxIter, option.limem, option.debug, ...
        option.tau, option.rho);
    tstop=toc(tstart);
    disp(['Time for JointMC7 is ', num2str(tstop)]);
    output.omega=omega;
    output.mean=img_mean;
    output.std=img_std;
elseif strcmpi(dataset, 'Multi-PIE')
    % to deal with multiple poses, where we generate one common part for
    % each pose
    prefix={'*illum_05*.jpg', '*illum_07*.jpg', '*illum_09*.jpg',...
        '*illum_27*.jpg', '*illum_29*.jpg', '*light_05*.jpg',...
        '*light_27*.jpg'};
    if option.debug==1
        f1=fopen('filelist.txt', 'w');
        fprintf(f1, 'People\tCondition\tFile\n');
    end
    cd(path);
    C_hat=zeros(option.height, option.width*option.n_People*length(prefix));
    A_hat=zeros(option.height, option.width*length(dir('*.jpg'))/option.n_People);
    E_hat=zeros(option.height*option.n_People,...
        option.width*length(dir('*.jpg'))/option.n_People);
    output=[];
    offset=0;
    for k=1: length(prefix)
        dirlist=dir(prefix{k});
        img_mean=zeros(option.n_People, option.n_Condition);
        img_std=ones(option.n_People, option.n_Condition);
        n_Condition=length(dirlist)/option.n_People;
        % deciding the sampling entries
        omega=GenerateSample(option.n_People, n_Condition, option.ratio);
        % load in the images
        X=zeros(option.height*option.n_People, option.width*n_Condition);
        for i=1: option.n_People
            for j=1: n_Condition
                if omega(i, j)==1
                    img=imread(dirlist((i-1)*n_Condition+j).name);
                    if size(img, 3)~=1
                        img=rgb2gray(img);
                    end
                    img=imresize(img, [option.height option.width]);
                    img=double(img);
                    % normalization needed
                    switch option.mode
                        case 0
                            % use the logrithm
                            img=log(img+1);
                        case 1
                            % unit l2-norm for each image
                            img_std(i, j)=norm(img, 'fro');
                            img=img/img_std(i, j);
                        case 2
                            % zero mean and unit l2-norm for each image
                            img_mean(i, j)=mean(img(:));
                            img=img-img_mean(i, j);
                            img_std(i, j)=norm(img, 'fro');
                            img=img/img_std(i, j);
                        case 3
                            % zero mean and unit std for eac dimension
                            img_std(i, j)=255;
                            img=img/255;
                            img_mean(i, j)=mean(img(:));
                            img=img-img_mean(i, j);
                        case 4
                            % zerp mean
                            img_mean(i, j)=mean(img(:));
                            img=img-img_mean(i, j);
                    end
                    X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=img;
                else
                    X((i-1)*option.height+1: i*option.height, (j-1)*option.width+1: j*option.width)=0;
                end
                % for debug only
                if option.debug==1
                    fprintf(f1, '%d\t\t%d\t\t%s\n', i, j, dirlist((i-1)*n_Condition+j).name);
                end
            end
        end
    %     matrix2im(X, 'YaleB-X.tif', 8);
        % start training
        % lambda=1/sqrt(size(X, 1));
        % use a smaller weight for sparisity term
        lambda=option.lambda/sqrt(option.height);
        % for debug only
        tstart=tic;
        % start program
        [C_hat(:, (k-1)*option.width*option.n_People+1: k*option.width*option.n_People),...
            A_hat(:, offset*option.width+1: (offset+n_Condition)*option.width),...
            E_hat(:, offset*option.width+1: (offset+n_Condition)*option.width),...
            output_tmp]=JointMC7(X, [option.height, option.width, ...
            option.n_People, n_Condition], omega, lambda, option.tol, ...
            option.maxIter, option.limem, option.debug, ...
            option.tau, option.rho);
        tstop=toc(tstart);
        disp(['Time for JointMC7 is ', num2str(tstop)]);
        offset=offset+n_Condition;
        output_tmp.omega=omega;
        output_tmp.mean=img_mean;
        output_tmp.std=img_std;
        output=[output output_tmp];
        output(1).omega=[output(1).omega omega];
    end
    if option.debug==1
        fclose(f1);
        clear f1;
    end
    option.n_People=option.n_People*length(prefix);
end
end
